// client.h
// $Id: client.h,v 1.44 1998/10/14 00:30:33 jyotaro Exp $

#ifndef __CLIENT_H__
#define __CLIENT_H__

#ifndef SJIS
#define EUC
#endif

#ifdef SJIS
#define iskanji(a) issjis(a)
#else
#define iskanji(a) iseuc(a)
#endif

struct BOARDPROPERTY {
	int bnum;
	char sname[11];
	char lname[32];
	int rlevel;
	int wlevel;
	bool bCug;
	BOARDPROPERTY(){ bnum = rlevel = wlevel = 0; sname[0] = lname[0] = '\0';}
};

/*
typedef struct tagARTICLEPROPERTY {

} ARTICLEPROPERTY;
*/

typedef struct tagLOGTIME {
	time_t ptime;
	time_t ltime;
	time_t lltime;
	time_t ctime;
} LOGTIME;

typedef struct tagREGISTRY {
	char key[BUFSIZE], value[BUFSIZE];
} REGISTRY;

struct USERINFO {
	int level;
	char id[IDLEN];
	char handle[HANDLELEN];
	char memo[MEMOLEN];
	USERINFO(){ level = 0; id[0] = handle[0] = memo[0] = '\0';}
};

class Client : public BBSSocket {
protected:
	short maxchannel, channel;
	char guestid[IDLEN];
	bool ImGuest;
	char clientname[32];
public:
	Client(int sd);
	virtual ~Client();
	virtual const char* GetGuestID() { return guestid; }
	virtual int Login(const char* id, const char* passwd_or_handle);
	virtual void Logout() { sendsz("QUIT\n"); }
	virtual int Auth(const char* id, const char* passwd_or_handle);
#ifndef DONTHAVE_UNIXSTDIO
	virtual void ReceiveTelegram() {;}		// Override it!
	virtual char* sreadline(const char* prompt, char* buf, int size, unsigned short mode = 0);
	virtual bool printmessage(int prefix, int exeption = 0);
	virtual void printversion() { sendsz("VERSION\n"); printmessage(36);};
	virtual void printopening() { sendsz("OPENING\n"); printmessage(19);};
	virtual void printsysinfo() { sendsz("SYSINFO\n"); printmessage(20);};
	virtual bool Getutmp(struct utmp* u);
#endif

// --- basic functions ---

	virtual bool waitfor(int prefix, int exception = 0, int exception2 = 0);
	virtual bool TrueOrFalse(const char* pt);
	virtual bool SwitchToSJIS();
	virtual bool SystemNews(string& news);
	virtual void Quit();

// --- user envirinment functions ---

	virtual void SetWhat(const char* buf);
	virtual bool SetPassword(const char* oldpass,const char* newpass);
	virtual bool SetHandle(const char* newhandle);
	virtual bool SetMemo(const char* memo);
	virtual bool Settty(const char* tty, const char* dev, const char* cname);

// --- channel functions ---

	short GetMaxChannel() { return maxchannel; }
	short GetCurrentChannel() { return channel; }
	virtual bool GetChannelInfo(int ch, CHINFO& info);
	virtual int GetChannelInfoVector(vector<CHINFO>& info);
	virtual int GetUserInfoVector(vector<USERINFO>& info);

// --- Telegram functions ---

	virtual bool SendTelegram(CHINFO* info, const char* msg);
//	virtual bool ReceiveTelegram(CHINFO* info, char* msg);

// --- time functions ---

	virtual bool SetLastTime(time_t ltime = 0);
	virtual bool SetLastTimeLocal(int bnum = 0, time_t ltime = 0);
	virtual time_t GetLastTime();
	virtual time_t GetLastTimeLocal(int bnum = 0);
	virtual void GetLogTime(LOGTIME& lt);

// --- registry functions ---

	virtual bool SetRegistryRoot(const char* rootkey);
	virtual bool SetRegistryValue(const char* key, const char* value);
	virtual bool GetRegistryValue(const char* key, char* value, int nlen);
	virtual bool GetRegistryValue(REGISTRY* rg, int num);
	virtual bool SetRegistryBool(const char* key, bool b);
	virtual bool GetRegistryBool(const char* key, bool bDefault = false);
	virtual bool SetRegistryInt(const char* key, int n);
	virtual int GetRegistryInt(const char* key, int nDefault = 0);
	virtual string GetRegistryString(const char* key, const char* Default = "");

// --- Message board functions ---

	virtual bool GetMessageBoardInfo(BOARDPROPERTY* brd, int bnum = 0);
	virtual int GetMessageBoardVector(vector<BOARDPROPERTY>& board);
	virtual bool MessageBoardFirst(BOARDPROPERTY* brd = NULL);
	virtual bool MessageBoardNext(BOARDPROPERTY* brd = NULL);
	virtual bool SetCurrentMessageBoard(int bnum, BOARDPROPERTY* brd = NULL);
	virtual int GetNumberOfNewArticle(int bnum = 0);
	virtual bool GetArticleProperty(int bnum, int anum, ARTICLEPROPERTY* mp);
	virtual int GetArticlePropertyVector(int bnum, int anum, vector<ARTICLEPROPERTY>& mp);

	virtual bool GetArticle(int bnum, int anum, string& article, ARTICLEPROPERTY* ap = NULL);
	virtual bool GetNewArticle(int bnum, string& article, ARTICLEPROPERTY* ap = NULL);
	virtual bool GetNextArticle(string& article, ARTICLEPROPERTY* ap = NULL);

	virtual bool EnterArticle(int bnum, const char* subject, const char* article);
	virtual bool KillArticle(int anum, int bnum = 0);

// --- File board functions ---

	virtual bool UploadFile(const char* localname, const char* subj, const char* filename, const char* dsc, int bnum = 0);
	virtual bool SymlinkFile(const char* localname, const char* subj, const char* filename, const char* dsc, int bnum = 0);

// --- Freetalk functions ---

	virtual bool GetFreetalkBoard(BOARDPROPERTY& board, int bnum);
	virtual int GetFreetalkBoardVector(vector<BOARDPROPERTY>& board);
	virtual int GetFreetalkBoardNum();
	virtual int GetFreetalkNumOfNewArticle(int bnum);
	virtual bool GetFreetalkArticle(int bnum, int anum, int code);
	virtual bool EnterFreetalkArticle(int bnum, const char* article);
	virtual bool SetFreetalkLastTime(int bnum, time_t ltime = 0);
	virtual bool EditFreetalkArticle(int bnum, int anum, const char* article);
	virtual void ShowFreetalkArticle(int bnum, int anum, char* id, char* handle, char* article, time_t atime, int code){;}; // Override It!
};

#define LOGIN_SUCCESS			0
#define LOGIN_INVALIDPASSWORD	1
#define LOGIN_CHANNELFULL 		2
#define LOGIN_UNKNOWNERROR		10

#ifndef DONTHAVE_UNIXSTDIO

#define SREADLINE_ECHOOFF	1
#define SREADLINE_PASSWD	2
#define SREADLINE_CHAT		4
#define SREADLINE_TELEGRAM	8

#define NEWLINE() putchar('\n')

#endif

int ConnectToService(const char* host = NULL, int port = 25999);

#endif // __CLIENT_H__
