// bbsd.h
// $Id: bbsd.h,v 1.25 1998/10/14 00:30:34 jyotaro Exp $

#ifndef __BBSD_H__
#define __BBSD_H__

static const char VERSION[][82] = {
	"STBBS Server version TEST ("__DATE__" "__TIME__")",
	"Programmed by T.Shimada <shima@cup.com>",
	"\0"
};

/***************************************************************************
  Definition of class Mutex
 ***************************************************************************/
class Mutex {
	pthread_mutex_t mutex;
public:
	Mutex() 
	{ 
		pthread_mutex_init(&mutex, NULL);
	}
	virtual ~Mutex() { ; }
	void Lock() { pthread_mutex_lock(&mutex); }
	void Unlock() { pthread_mutex_unlock(&mutex); }
	operator pthread_mutex_t*() { return &mutex; }
};

/***************************************************************************
  Definition of class Channel (Parent : BBSSocket)
 ***************************************************************************/
class Channel : public BBSSocket {
//	char what[WHATLEN], memo[MEMOLEN], tty[TTYLEN],
//	CHINFO information;
	char telegramqueue[800], regroot[BUFSIZE];
	int currentboard, currentarticle, currentid;
	int currentfboard;
	unsigned int chatroom;
	time_t logintime;
	Mutex SendLock, TelegramLock;
	pthread_t self;
	bool bAuthorized;
public:
	bool bSamehost;

	Channel(int sd);
	virtual ~Channel();
//	char* GetUserID() { return information.id; }
//	char* GetHandle() { return information.handle; }
//	char* GetWhat() { return information.what /*what*/; }
//	char* GetMemo() { return information.memo /*memo*/; }
	pthread_t GetThreadHandle() { return self; }
	bool IsInChat(int ch = 0) { return (chatroom >> ch) & 1 ? true : false; }
	void version();
	void Authorize();
	void Login();
	void whois();
	void idwhois();
	void whoami();
	void wholist();
	void userlist();
	void setwhat();
	void setmemo();
	void settty();
	void telegram();
	void receivetelegram();
	bool AddTelegramQueue(const char* fmt, ...);
	void quit();
	void boardlist();
	void setboard();
	void boardfirst();
	void boardnext();
	void articlelist();
	FILE* OpenProfile(char* profname, char* accessmode);
	int ReadProfileInt(const char* profname, int ndefault = 0);
	string ReadProfileString(const char* profname, const char* szdefault = "");
	bool WriteProfileInt(const char* profname, int val);
	bool WriteProfileString(const char* profname, const char* buf);
	bool ShowMessageProperty(int bnum, int anum);
	bool ShowArticle(int bnum, int anum);
	bool ShowBoardProp(int bnum = 0);
	void readarticle();
	void readnextarticle();
	void numberofnewarticle();
	void readfirstnewarticle();
	void currentarticleprop();
	void articleprop();
	void enterarticle();
	void killarticle();
	
	// File board functions
	bool ShowFileBoardProp(int bnum = 0);
	void fboardlist();
	void setfboard();
	void fboardfirst();
	void fboardnext();
	void newfilelist();
	void filelist();
	void getfiledescription();
	void getfile();
	void putfile();
	void setlastftime();
	void fsymlink();	
	bool ShowFileProperty(FILEPROPERTY& mp);
	bool ShowFileProperty(int bnum, int anum);
	bool ShowFileDescription(int bnum, int anum);
	void fileattention();
	
	// Registry functions
	void setregroot();
	void getregvalue();
	void setregvalue();

	void SendChatSaying(const char* buf);
	void chat();
	void enterroom();
	void exitroom();
	void say();
	
	void setlasttime();
	void getlasttime();
	void setlasttimelocal();
	void getlasttimelocal();
	void apply();
	void sendmail();
	void readmail();
	void enterprofile();
	void readprofile();
	void readprofileex();
	void setshiftjis();
	void kick();
	void gettime();
	void last();
	void sethandle();
	void setpassword();
	virtual char* recvln(unsigned int flags = 0);
	virtual string& recvln(string& str, unsigned int len = 0);
	void TalkToMonitor();
	void Run();
#ifdef FREETALK
	void ft_whatboardis();
	void ft_boardlist();
	void ft_readarticle();
	void ft_enterarticle();
	void ft_getboardnum();
	void ft_numberofnewarticle();
	void ft_setlasttime();
	void ft_getlasttime();
	void ft_editarticle();
#endif
};

/***************************************************************************
  Type definitions of bbsd
 ***************************************************************************/
typedef struct tagLOGRECORD {
	char id[9], handle[15], tty[12];
	int channel;
	time_t itime, otime;
} LOGRECORD;

/***************************************************************************
  Global prototypes of utilities & misc functions
 ***************************************************************************/
int ReadProfileInt(const char* profname, const char* targetid, int ndefault = 0);
string ReadProfileString(const char* profname, const char* targetid, const char* szdefault = "");
bool WriteProfileInt(const char* profname, const char* targetid, int val);
bool WriteProfileString(const char* profname, const char* targetid, const char* buf);

bool IsFileExist(const char* fname);
bool IsDirectoryExist(const char* fname);
/***************************************************************************
  Global variables as external.
 ***************************************************************************/
extern Channel* channel[MAX_CHANNEL];
extern Hash GlobalConfig;					// Global configurations ( bbs.conf )


/***************************************************************************
  Global macro definitions of bbsd
 ***************************************************************************/
#define CONFIG_IDPREFIX				(GlobalConfig.GetValue("idprefix", IDPREFIX).c_str())
#define CONFIG_BBSROOT				(GlobalConfig.GetValue("bbsroot", BBSROOT).c_str())
#define CONFIG_HIDESYSOPLOG			((bool)GlobalConfig.GetValue("hidesysoplog", (int)false))
#define CONFIG_DSPSOUND				((bool)GlobalConfig.GetValue("dspsound", (int)false))
#define CONFIG_ENABLECHANGEHANDLE	((bool)GlobalConfig.GetValue("enablechangehandle", (int)true))
#define CONFIG_BEEP					((bool)GlobalConfig.GetValue("beep", (int)false))
#define CONFIG_PORT					(GlobalConfig.GetValue("port", PORT))
#define CONFIG_GUESTID				(GlobalConfig.GetValue("guestid", GUESTID).c_str())
#define CONFIG_ADMIN				(GlobalConfig.GetValue("admin", ADMIN).c_str())
#define CONFIG_SYSOPLEVEL			(GlobalConfig.GetValue("sysoplevel", SYSOP_LV))
#define CONFIG_AUPLAYER				(GlobalConfig.GetValue("auplayer", AUPLAYER).c_str())
#define CONFIG_WAVPLAYER			(GlobalConfig.GetValue("wavplayer", WAVPLAYER).c_str())
#define CONFIG_SYSOPCALLSOUND		(GlobalConfig.GetValue("sysopcallsound", SYSOPCALLSOUND).c_str())
#define CONFIG_LOGINSOUND			(GlobalConfig.GetValue("loginsound", LOGINSOUND).c_str())
#define CONFIG_LOGOUTSOUND			(GlobalConfig.GetValue("logoutsound", LOGOUTSOUND).c_str())
#define CONFIG_SENDMAIL				(GlobalConfig.GetValue("sendmail", SENDMAIL).c_str())

#endif // __BBSD_H__
