#include <pthread.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <time.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <string>		// ANSI string class
#include <stl.h>		// ANSI Standard Class Library

#include "stbbs.h"
#include "config.h"
#include "bbsd.h"

Mutex FileLock;

////////////////////////////////////////////// File board functions

bool GetFileBoardName(int bnum, char sname[11], char lname[32])
{
	char buf[MAXPATH];
	FILE* f;
	sprintf(buf, FILEDIR, bnum);
	strcat(buf, "boardname");
	f = fopen(buf, "rt");
	if (!f) return false;
	if (sname) {
		fgets(sname, 11, f);
		sname[strlen(sname) - 1] = '\0';
	}
	if (lname) {
		fgets(lname, 32, f);
		lname[strlen(lname) - 1] = '\0';
	}
	fclose(f);
	return true;
}

int GetNextFileNumber(int bnum /* =0 */)
{
	char buf[256];
	sprintf(buf, FILEDIR"next", bnum);
	FILE* f = fopen(buf, "r");
	if (!f) return 1;		// If file 'next' not exist, return 1 as default value.
	int next = atoi(fgets(buf, sizeof(buf), f));
	fclose(f);
	return next;
}

void IncrementNextFileNumber(int bnum)
{
	int next = GetNextFileNumber(bnum);

	char buf[256];
	sprintf(buf, FILEDIR"next", bnum);
	
	FILE* f = fopen(buf, "w");
	if (!f) { return; }
	fprintf(f, "%d", next + 1);
	fclose(f);
}

void Channel::fboardlist()
{
	for (int i = 1; i < MAX_FILEBOARD; i++) {
		char sname[11], lname[32];
		if (GetFileBoardName(i, sname, lname)) {
			sendsz("22-%d\t%s\t%s\n", i, sname, lname);
		}
	}
	sendsz("18-EOM\n");
}

void Channel::setfboard()
{
	sendsz("23-Enter board number\n");
	int bnum = atoi(recvln());
	if (bnum != 0) currentfboard = bnum;
	if (!ShowFileBoardProp(currentfboard)) {
		sendsz("42-No such board.\n");
	}
}

void Channel::fboardfirst()
{
	int bnum;
	for (bnum = 1; ShowFileBoardProp(bnum) == false && bnum < MAX_FILEBOARD; bnum++) ; // bacant
	if (bnum == MAX_FILEBOARD) {
		sendsz("42-No such board.\n");
		currentfboard = 0;
		return;
	}
	currentfboard = bnum;
}

void Channel::fboardnext()
{
	int bnum;
	for (bnum = currentfboard + 1; ShowFileBoardProp(bnum) == false && bnum < MAX_FILEBOARD; bnum++) ; // bacant
	if (bnum == MAX_FILEBOARD) {
		sendsz("42-No such board.\n");
		currentfboard = 0;
		return;
	}
	currentfboard = bnum;
}

bool GetFileProperty(int bnum, int anum, FILEPROPERTY* mp)
{
	mp->bnum = bnum;
	mp->anum = anum;
	char tmp[11];
	GetFileBoardName(bnum, tmp, mp->bname);
	
	char buf[MAXPATH];
	sprintf(buf, FILEDIR"mess%d", bnum, anum);

	FILE* f = fopen(buf, "r");
	if (!f) return false;
	fgets(mp->senderid, sizeof(mp->senderid), f);
	mp->senderid[strlen(mp->senderid) - 1] = '\0';
	fgets(mp->subject, sizeof(mp->subject), f);
	mp->subject[strlen(mp->subject) - 1] = '\0';
	fgets(mp->filename, sizeof(mp->filename), f);
	mp->filename[strlen(mp->filename) - 1] = '\0';
	fclose(f);

	strcpy(mp->senderhandle, ReadProfileString("handle", mp->senderid).c_str());

	// Get status of a file
	struct stat s;
	stat(buf, &s);
	mp->postedtime = s.st_ctime;
	
	struct tm* t;
	t = localtime(&mp->postedtime);
	
	sprintf(mp->date, "%02d/%02d/%02d", t->tm_year % 100, t->tm_mon + 1, t->tm_mday);
	sprintf(mp->time, "%02d:%02d:%02d", t->tm_hour, t->tm_min, t->tm_sec);
	
	// Get size of binary file.
	sprintf(buf, FILEDIR"file%d.bin", bnum, anum);
	if (stat(buf, &s) != 0) s.st_size = 0;
	mp->size = s.st_size;
	
	// Get Download count.
	mp->getcount = 0;
	sprintf(buf, FILEDIR"getlog%d", bnum, anum);
	f = fopen(buf, "r");
	if (f) {
		int c;
		while ((c = fgetc(f)) != EOF) if ((char)c == '\n') mp->getcount++;
		fclose(f);
	}
	return true;
}

bool Channel::ShowFileProperty(FILEPROPERTY& mp)
{
	sendsz("70-%d\t%s\t%d\t%d\t%s\t%s\t%s\t%s\t%s\n", 
		mp.anum, mp.filename, mp.size, mp.getcount, 
		mp.date, mp.time, mp.senderid, mp.senderhandle, mp.subject);
	return true;
}

bool Channel::ShowFileProperty(int bnum, int anum)
{
	if (bnum == 0) bnum = currentfboard;
	FILEPROPERTY mp;
	if (!GetFileProperty(bnum, anum, &mp)) return false;
	return ShowFileProperty(mp);
}

void Channel::newfilelist() 
{
	sendsz("23-Enter board number (0 to current)\n");
	int bnum = atoi(recvln());
	if (bnum == 0) bnum = currentfboard;
	time_t ltime = ReadProfileInt("lftime", time(NULL) - 60 * 60 * 24 * 10);
	int anum;
	FileLock.Lock();
	int newest = GetNextFileNumber(bnum) - 1;
	FileLock.Unlock();
	for (anum = newest; anum >= 1; anum--) {
		FILEPROPERTY mp;
		if (GetFileProperty(bnum, anum, &mp)) {
			if (mp.postedtime > ltime) {
				ShowFileProperty(mp);
			}
		}
	}
	sendsz("18-EOM\n");
}

void Channel::filelist()
{
	sendsz("23-Enter board number (0 to current)\n");
	int bnum = atoi(recvln());
	if (bnum == 0) bnum = currentfboard;
	sendsz("24-Enter first file number (0 to all)\n");
	int first = atoi(recvln());
	FileLock.Lock();
	int newest = GetNextFileNumber(bnum) - 1;
	FileLock.Unlock();
	if (first) {
		for (int i = first; i <= newest; i++) {
			ShowFileProperty(bnum, i);
		} 
	} else {
		for (int i = newest; i >= 0; i--) {
			ShowFileProperty(bnum, i);
		}
	}
	sendsz("18-EOM\n");
}

FILE* OpenDescriptionFile(int bnum, int anum, char* accessmode)
{
	char buf[256];
	sprintf(buf, FILEDIR"mess%d", bnum, anum);
	return fopen(buf, accessmode);
}

bool Channel::ShowFileDescription(int bnum, int anum)
{
	if (bnum == 0) bnum = currentfboard;
	FILEPROPERTY mp;
	if (!GetFileProperty(bnum, anum, &mp)) return false;

	char buf[256];
	FILE* f = OpenDescriptionFile(bnum, anum, "r");
	
	fgets(buf, sizeof(buf), f);	// skip sender id
	fgets(buf, sizeof(buf), f);	// skip subject
	fgets(buf, sizeof(buf), f);	// skip filename
	
	ShowFileProperty(mp);
	while (fgets(buf, sizeof(buf), f)) {
		sendsz("27-%s", buf);
	}
	sendsz("18-EOM\n");
	fclose(f);
	
	return true;
}

void Channel::getfiledescription()
{
	sendsz("23-Enter board number (0 to current)\n");
	int bnum = atoi(recvln());
	if (bnum == 0) bnum = currentfboard;
	FileLock.Lock();
	int next = GetNextFileNumber(bnum);
	FileLock.Unlock();
	sendsz("21-%d\n", next - 1);
	sendsz("26-Enter article number.\n");
	int anum = atoi(recvln());

	if (ShowFileDescription(bnum, anum)) {
		currentfboard = bnum;
	} else {
		sendsz("43-No such file.\n");
	}
}

void Channel::getfile()
{
	sendsz("23-Enter board number (0 to current)\n");
	int bnum = atoi(recvln());
	if (bnum == 0) bnum = currentfboard;
	int next = GetNextFileNumber(bnum);
	sendsz("21-%d\n", next - 1);
	sendsz("26-Enter article number.\n");
	int anum = atoi(recvln());
	
	char filename[MAXPATH];
	sprintf(filename, FILEDIR"file%d.bin", bnum, anum);
	struct stat s;
	
	bool bTemp = false;

	if (stat(filename, &s) != 0) {
		sprintf(filename, FILEDIR"file%d.sh", bnum, anum);
		FILE* f;
		if (stat(filename, &s) == 0 && (f = popen(filename, "rb")) != NULL && tmpnam(filename)) {
			// 륹ץȤä
			char buf[BUFSIZE];
			FILE* f2 = fopen(filename, "wb");
			int r;
			while ((r = fread(buf, 1, sizeof(buf), f)) > 0) {
				fwrite(buf, 1, r, f2);
			}
			pclose(f);
			fclose(f2);
			bTemp = true;
		} else {
			sendsz("43-No such file.\n");
			return;
		}
	}
	sendsz("71-Binary file transferring start.\n");
	sendfile(filename);
	if (bTemp) unlink(filename);
	
	// download log..
	FileLock.Lock();
	sprintf(filename, FILEDIR"getlog%d", bnum, anum);
	FILE* f = fopen(filename, "a");
	if (f) {
		time_t tim = time(NULL);
		fprintf(f, "%s %s %s %s\n", timetodate(tim).c_str(), timetotime(tim).c_str(), information.id, information.handle);
		fclose(f);
	}
	FileLock.Unlock();
	sendsz("8-Operation succeed.\n");
}

void Channel::putfile()
{
	sendsz("23-Enter board number (0 to current)\n");
	int bnum = atoi(recvln());
	if (bnum == 0) bnum = currentfboard;
	
	if (!ShowFileBoardProp(bnum)) {
		sendsz("42-No such fileboard.\n");
		return;
	}

	sendsz("33-Enter subject.\n");
	string subj = recvln();
	sendsz("74-Enter filename.\n");
	string filename = recvln();
	sendsz("30-Enter file description. // to terminate.\n");
	string description;
	while (strcmp(recvln(), "//") != 0) {
		description += lpbuf;
		description += "\n";
	}

	sendsz("72-Start to send binary file,\n");
	char tmpname[BUFSIZE];
	sprintf(tmpname, FILEDIR"file%d.tmp", bnum, SockDsc);
	if (!recvfile(tmpname)) {
		sendsz("31-File couldn't create.\n");
		return;
	} else {
		FileLock.Lock();
		// Write to description file
		int next = GetNextFileNumber(bnum);
		FILE* f = OpenDescriptionFile(bnum, next, "w");
		if (!f) {
			sendsz("31-File couldn't create.\n");
			FileLock.Unlock();
			return;
		}
		fprintf(f, "%s\n%s\n%s\n", information.id, subj.c_str(), filename.c_str());
		fwrite(description.c_str(), 1, description.length(), f);
		fclose(f);

		char buf[256];
		sprintf(buf, FILEDIR"file%d.bin", bnum, next);
		rename(tmpname, buf);

		// Increment file number and unlock mutex.
		IncrementNextFileNumber(bnum);
		FileLock.Unlock();
	}
	sendsz("32-File has posted successfully.\n");
}

bool Channel::ShowFileBoardProp(int bnum /* = 0 */)
{
	char sname[11], lname[32];
	if (bnum == 0) bnum = currentfboard;
	if (GetFileBoardName(bnum, sname, lname)) {
		sendsz("22-%d\t%s\t%s\n", bnum, sname, lname);
		return true;
	}
	return false;
}

void Channel::setlastftime()
{
	sendsz("44-Enter time by time_t format.  just enter to now.\n");
	recvln();
	time_t ltime;
	if (lpbuf[0]) {
		ltime = atoi(lpbuf);
	} else {
		ltime = time(NULL);
	}
	WriteProfileInt("lftime", ltime);
	sendsz("8-Operation succeed.\n");
}

void Channel::fsymlink()
{
	if (userlevel < 5) {
		sendsz("68-The operation is not permitted for you.\n");
		return;
	}
	sendsz("23-Enter board number (0 to current)\n");
	int bnum = atoi(recvln());
	if (bnum == 0) bnum = currentfboard;
	
	if (!ShowFileBoardProp(bnum)) {
		sendsz("42-No such fileboard.\n");
		return;
	}

	sendsz("33-Enter subject.\n");
	string subj = recvln();
	sendsz("74-Enter filename.\n");
	string filename = recvln();
	sendsz("73-Enter real full-pathname on server disk.\n");
	string realfile = recvln();
	

	sendsz("30-Enter file description. // to terminate.\n");
	string description;
	while (strcmp(recvln(), "//") != 0) {
		description += lpbuf;
		description += "\n";
	}

	FileLock.Lock();
	// Write to description file
	int next = GetNextFileNumber(bnum);
	
	FileLock.Lock();
	FILE* f = OpenDescriptionFile(bnum, next, "w");
	if (!f) {
		sendsz("31-File couldn't create.\n");
		FileLock.Unlock();
		return;
	}
	fprintf(f, "%s\n%s\n%s\n", information.id, subj.c_str(), filename.c_str());
	fwrite(description.c_str(), 1, description.length(), f);
	fclose(f);
	FileLock.Unlock();
	
	// Create symbolic link.
	char buf[256];
	sprintf(buf, FILEDIR"file%d.bin", bnum, next);
	symlink(realfile.c_str(), buf);
	
	// Increment file number and unlock mutex.
	IncrementNextFileNumber(bnum);
	FileLock.Unlock();
	sendsz("32-File has posted successfully.\n");
}

void Channel::fileattention()
{
	output_file(MESSAGEDIR"fileatn.jtx", 75);
}
