// jcode.cpp (based on nkf.c)

#include <stdio.h>
#include <sys/types.h>
#include <string>			// STL

#include "jcode.h"

#define		false	0
#define		true	1

#define		ESC	0x1b
#define		SP	0x20
#define		AT	0x40
#define		SSP	0xa0
#define		DEL	0x7f

#define		HOLD_SIZE	32

#define		DEFAULT_KI	'@'
#define		DEFAULT_KO	'J'
#define		DEFAULT_I	oconv		/* EUC, JIS7 */
// #define	DEFAULT_I	s_oconv		/* Shift JIS */

#define		SJ0162	0x00e1		/* 01 - 62 ku offset */
#define		SJ6394	0x0161		/* 63 - 94 ku offset */

/* buffers */
char			stdbuf[BUFSIZ];
short			hold_buf[HOLD_SIZE];
int				hold_count;

/* flags */
int				unbuf_f = true,
				estab_f = false,
				nop_f = false;

/* options */
char			k_in = DEFAULT_KI,
				k_out = DEFAULT_KO;

/* converters */
int				s_iconv (int, int);

int				e_oconv (int, int);
int				j_oconv (int, int);
int				s_oconv (int, int);

unsigned int		nops (char);

int				(*iconv) (int, int);
unsigned int		(*r1conv) (char);
unsigned int		(*r2conv) (char);
int				(*oconv) (int, int);

/* Global states */
int				in_kanji = false,
				k_shift = false;

string result;

int GETC(const char*& f)
{
	if (*f) {
		return (int)((unsigned char)(*f++));
	}
	return EOF;
}

void PUTCHAR(int c)
{
	if (c != EOF) result += (char)c;
}

int push_hold_buf (int c2, int c1)
{
	if (hold_count >= HOLD_SIZE) return (EOF);
	hold_buf[hold_count++] = ((c2 & 0x0ff) << 8) | (c1 & 0x0ff);
	return ((hold_count >= HOLD_SIZE) ? EOF : hold_count);
}

h_conv (const char*& f, int c2, int c1)
{
	int	   wc;

	/** it must NOT be in the kanji shifte sequence	 */
	/** it must NOT be written in JIS7			 */
	/** and it must be after 2 byte 8bit code		 */

	hold_count = 0;
	push_hold_buf (c2, c1);
	c2 = 0;

	while ((c1 = GETC (f)) != EOF) {
		if (c2) {
			/* second byte */
			if (c1 > DEL) {
			/* then 8th bit is on */
				if (!estab_f) {
					/* not established */
					if (c1 > SSP) {
						/* it is still ambiguious yet */
						/* goto send */
					} else if (c1 < AT) {
						/* ignore bogus code */
						c2 = 0;
						continue;
						/* goto next byte */
					} else {
						/* now established */
						/* it seems to be Shift JIS */
						estab_f = true;
						iconv = s_iconv;
						/* goto send */
					}
				} else;
					/* just established */
					/* goto send without checking */
			} else {
				/* 7 bit code */
				/* it might be bogus code  then ignore */
				c2 = 0;
				continue;
				/* goto next byte */
			}
		} else {
			/* First byte */
			if (c1 > DEL) {
				/* 8th bit is on */
				if (c1 < SSP) {
					/* it seems to be Shift JIS */
					estab_f = true;
					iconv = s_iconv;
				} else if (c1 < 0xe0) {
					/* it seems to be EUC */
					estab_f = true;
				} else {
					/* still ambiguious */
				}
				c2 = c1;
				continue;
				/* goto next_byte */
			} else;
			/* 7 bit code , then send without any process */
			/* goto send */
		}
		/* send: */
		if ((push_hold_buf (c2, c1) == EOF) || estab_f) break;
		c2 = 0;
		continue;
	}

	/** now,
	 ** 1) EOF is detected, or
	 ** 2) Code is established, or
	 ** 3) Buffer is FULL (but last word is pushed)
	 **
	 ** in 1) and 3) cases, we must use default
	 ** Kanji codes, that is DEFAULT_I
	 **/

	if (estab_f == false) {
		iconv = DEFAULT_I;
		estab_f = true;
	}
	for (wc = 0; wc < hold_count; wc++) {
		c1 = hold_buf[wc];
		c2 = 0x0ff & (c1 >> 8);
		c1 = c1 & 0x0ff;
		(*iconv) (c2, c1);
	}
	return 0;
}

convert (const char*& f)
{
	int c1, c2;

	c2 = 0;
	iconv = oconv;

	while ((c1 = GETC (f)) != EOF) {
		if (c2) {
			/* second byte */
			if (c2 > DEL) {
				/* in case of 8th bit is on */
				if (!estab_f) {
					/* in case of not established yet */
					if (c1 > SSP) {
						/* It is still ambiguious */
						h_conv (f, c2, c1);
						c2 = 0;
						continue;
						/* goto next_word */
					} else if (c1 < AT) {
						/* ignore bogus code */
						c2 = 0;
						continue;
						/* goto next_word */
					} else {
						/* established */
						/* it seems to be Shift JIS */
						estab_f = true;
						iconv = s_iconv;
						/* goto send */
					}
				} else {
					/* in case of already established */
					if (c1 < AT) {
						/* ignore bogus code */
						c2 = 0;
						continue;
						/* goto next_word */
					} else;
					/* goto send */
				}
			} else {
				/* 7 bit code */
				/* it might be kanji shitfted */
				if ((c1 == DEL) || (c1 <= SP)) {
					/* ignore bogus code */
					c2 = 0;
					continue;
					/* goto next_word */
				} else;
			}
			/* goto send */
		} else {
			/* first byte */
			if (c1 > DEL) {
				/* 8 bit code */
				if (!estab_f) {
					/* not established yet */
					if (c1 < SSP) {
						/* it seems to be Shift JIS */
						estab_f = true;
						iconv = s_iconv;
					} else if (c1 < 0xe0) {
						/* it seems to be EUC */
						estab_f = true;
					} else {
						/* still ambiguious */
					}
				} else {
					/* already established */
				}
				c2 = c1;
				continue;
				/* goto next_byte */
			} else if ((c1 > SP) && (c1 != DEL)) {
				/* in case of Roman characters */
				if (k_shift) {
					/* in case of Kanji shifted */
					c2 = c1;
					continue;
					/* goto next_byte */
				} else;
				/* goto send */
			} else if (c1 == ESC) {
				/* in case of ESC */
				if ((c1 = GETC (f)) == EOF) {
					(*iconv) (0, ESC);
					break;
				} else if (c1 == '$') {
					if ((c1 = GETC (f)) == EOF) {
						(*iconv) (0, ESC);
						(*iconv) (0, '$');
						break;
					} else {
						/* in case of Kanji in ESC sequence */
						k_shift = true;
						continue;
					}
				} else if (c1 == '(') {
					if ((c1 = GETC (f)) == EOF) {
						(*iconv) (0, ESC);
						(*iconv) (0, '(');
						break;
					} else {
						/* in case of Kanji out ESC sequence */
						k_shift = false;
						continue;
					}
				} else {
					/* not kanji control */
					(*iconv) (0, ESC);
					/* goto send */
				}
			} else;
			/* goto send */
		}
		/* send: */
		(*iconv) (c2, c1);
		c2 = 0;
		continue;
		/* goto next_word */
	}

	/* epilogue */
	(*iconv) (EOF, 0);
	return 0;
}

s_iconv (int c2, int c1)
{
	if ((c2 == EOF) || (c2 == 0)) {
		/* NOP */
	} else {
		c2 &= 0x0ff;
		c1 &= 0x0ff;

		c2 = c2 + c2 - ((c2 <= 0x9f) ? SJ0162 : SJ6394);
		if (c1 < 0x9f) {
			c1 = c1 - ((c1 > DEL) ? SP : 0x1f);
		} else {
			c1 = c1 - 0x7e;
			c2++;
		}
	}
	(*oconv) (c2, c1);
}

e_oconv (int c2, int c1)
{
	if (c2 == EOF) return 0;
	else if (c2 == 0) PUTCHAR ((*r1conv) (c1));
	else {
		PUTCHAR (((*r2conv) (c2)) | 0x080);
		PUTCHAR (((*r2conv) (c1)) | 0x080);
	}
}

s_oconv (int c2, int c1)
{
	if (c2 == EOF) return 0;
	else if (c2 == 0) PUTCHAR ((*r1conv) (c1));
	else {
		c2 = (*r2conv) (c2);
		c1 = (*r2conv) (c1);
		PUTCHAR ((((c2 - 1) >> 1) + ((c2 <= 0x5e) ? 0x71 : 0xb1)));
		PUTCHAR ((c1 + ((c2 & 1) ? ((c1 < 0x60) ? 0x1f : 0x20) : 0x7e)));
	}
}

j_oconv(int c2, int c1)
{
	if (c2 == EOF) {
		if (in_kanji) {
			PUTCHAR (ESC);
			PUTCHAR ('(');
			PUTCHAR (k_out);
		}
	} else if (c2 == 0) {
		if (in_kanji) {
			PUTCHAR (ESC);
			PUTCHAR ('(');
			PUTCHAR (k_out);
			in_kanji = false;
		}
		PUTCHAR ((*r1conv)(c1));
	} else {
		if (in_kanji == false) {
			PUTCHAR (ESC);
			PUTCHAR ('$');
			PUTCHAR (k_in);
			in_kanji = true;
		}
		PUTCHAR ((*r2conv)(c2));
		PUTCHAR ((*r2conv)(c1));
	}
}

unsigned int nops(char c)
{
	return (c & 0x07f);
}

string kanjiconvert(const char* str, int kcode)
{
	unbuf_f = true;
	estab_f = false;
	nop_f = false;
	in_kanji = false;
	k_shift = false;
	result = "";
	r1conv = nops;
	r2conv = nops;

	switch (kcode) {
	case KANJI_JIS: oconv = j_oconv; break;
	case KANJI_EUC: oconv = e_oconv; break;
	case KANJI_SJIS: oconv = s_oconv; break;
	default: break;
	}

	convert(str);
	return result;
}

/*
int main()
{
	string a = kanjiconvert("ؤؤФäۤˤۤۤ", KANJI_SJIS);
	printf("%s\n", a.c_str());
	return 0;
}
*/
