#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <stdlib.h>
#include <netdb.h>
#include <netinet/in.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <sys/ioctl.h>
#include <sys/termios.h>
#include <sys/socket.h>
#include <string>
#include <stl.h>

#ifdef __FreeBSD__
#include <sys/select.h>
#include <sys/filio.h>
#include <sys/snoop.h>
#define std_in 0
#define std_out 1
#define TCGETA TIOCGETA
#define TCSETA TIOCSETA
#define termio termios
#endif

#include "stbbs.h"
#include "config.h"

#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/shm.h>

#define MSGSZ 128
#define BBSC	"bbsc"
#define CLEAR printf("\x1b[H\x1b[J")

typedef struct tagMSGBUF {
	long mtype;
	char mtext[MSGSZ];
} MSGBUF;

void PrintAll(CHINFO* channel)
{
	CLEAR;
	puts("  *** STBBS Channel monitor ***");
	puts("------------------------------------------------------------\n");
	for (int i = 0; i < MAX_CHANNEL; i++) {
		if (channel[i].id[0]) {
			printf("CH%02d: %-8s %-14s %-8s %s\n", i, channel[i].id, 
				channel[i].handle, channel[i].what, channel[i].memo);
		} else {
			printf("CH%02d: 󤵤Ƥޤ\n", i);
		}
		for (int j = 0; j < 19 / MAX_CHANNEL - 1; j++) putchar('\n');
	}
	puts("------------------------------------------------------------");
	puts("  ESC : ");
	puts("  ^C  :  λ");
}

bool snoop(const char* dev)
{
#if defined(__FreeBSD__) && defined(SNOOP)
// ꤫
	int snp_io, nread;
	struct stat sb;
	dev_t snp_tty;
	fd_set fd_s;
	char buf[1024];

	CLEAR;
	if ((snp_io = open("/dev/snp0", O_RDONLY)) < 0)
		return false;

	stat(dev,&sb);
	snp_tty = sb.st_rdev;

	if (ioctl(snp_io, SNPSTTY, &snp_tty) != 0) {
		close(snp_io);
		return false;
	}
	while(1) {
		FD_ZERO(&fd_s);
		FD_SET(std_in, &fd_s);
		FD_SET(snp_io, &fd_s);
		select(snp_io + 1, &fd_s, NULL, NULL, NULL);
		if (FD_ISSET(std_in, &fd_s)) {
			if (ioctl(std_in, FIONREAD, &nread) != 0)
				return false;
			switch (tolower(fgetc(stdin))) {
			case ' ': goto snpend;
//			case 'e': //EDIT
			}
		}
		if (FD_ISSET(snp_io, &fd_s)) {
			if (ioctl(snp_io, FIONREAD, &nread) != 0)
				return false;
			read(snp_io, buf, nread);
			write(std_out, buf,nread);
		}
	}
snpend:
		close(snp_io);
	return true;
#else
	return false;
#endif // defined(__FreeBSD__) && defined(SNOOP)
}

void Monitor()
{
	termio tio, tio_orig;
	ioctl(0, TCGETA, &tio);
	tio_orig = tio;
	tio.c_lflag &= ~ECHO;
	tio.c_lflag &= ~ICANON;
	tio.c_cc[VMIN] = 1;
	tio.c_cc[VTIME] = 0;
	ioctl(0, TCSETA, &tio);

	int shmid = shmget(PORT, sizeof(CHINFO) * MAX_CHANNEL, 0666);
	if (shmid < 0) { perror("Monitor:shmget"); exit(1); }
	CHINFO* channel = (CHINFO*)shmat(shmid, NULL, 0);
	if ((int)channel < 0) { perror("Monitor:shmat"); exit(1); }
	PrintAll(channel);

	CHINFO channelbak[MAX_CHANNEL];

	for (;;) {
		memcpy(channelbak, channel, sizeof(channelbak));
		fd_set fdset;
		FD_ZERO(&fdset);
		FD_SET(0, &fdset);
		timeval tv = { 1, 0 };
		select(1, &fdset, NULL, NULL, &tv);
		if (FD_ISSET(0, &fdset)) {
			switch (tolower(fgetc(stdin))) {
			case '\x1b': goto end;
//			case 'h': //SHOW HELP
			case ' ':
				PrintAll(channel);
				break;
			case '0':
				snoop(channel[0].dev);
				PrintAll(channel);
				break;
			}
		}

		int i;
		for (i = 0; i < MAX_CHANNEL; i++) {
			if (strcmp(channelbak[i].id, channel[i].id) != 0) break;
			if (strcmp(channelbak[i].handle, channel[i].handle) != 0) break;
			if (strcmp(channelbak[i].what, channel[i].what) != 0) break;
			if (strcmp(channelbak[i].memo, channel[i].memo) != 0) break;
		}
		if (i == MAX_CHANNEL) continue;
		PrintAll(channel);
	}
end:

	shmdt((char*)channel);
	ioctl(0, TCSETA, &tio_orig);
	CLEAR;
}

int main()
{
	setvbuf(stdout, NULL, _IONBF, 0);
	setvbuf(stdin, NULL, _IONBF, 0);
	
	for (;;) {
		Monitor();
		system(BBSC);
	}
	return 0;
}

