// stl-html.cpp
#include <stdio.h>
#include <stdarg.h>
#include <string>
#include <stl.h>
#include "stl-html.h"

#define BUFSIZE 32768

static string format(const char* fmt, ...)
{
	char buf[BUFSIZE];
	va_list ap;
	va_start(ap, fmt);
	vsprintf(buf, fmt, ap);
	va_end(ap);
	
	return buf;
}

void begin_html(const string title, const string urltojump = "", int sec = 1)
{
	printf("Content-type: text/html\n\n");
	printf("<!DOCTYPE HTML PUBLIC \"-//IETF//DTD HTML//EN\">\n");
	if (urltojump != "") printf("<meta http-equiv=refresh content=\"%d; url=%s\">", sec, urltojump.c_str());
	puts("<HTML>\n");
	puts("<HEAD>\n");
	puts("<meta http-equiv=\"Content-Type\" content=\"text/html; charset=x-euc-jp\">");
	printf("<TITLE>%s</TITLE></HEAD>\n", title.c_str());
}

void end_html()
{
	printf("</HTML>\n");
}

void begin_body(const string& options = "")
{
	if (options != "") {
		printf("<BODY %s>\n", options.c_str());
	} else {
		puts("<BODY>");
	}
}

void end_body()
{
	printf("</BODY>\n");
}

void begin_form(const string& action)
{
	printf("<FORM ACTION=\"%s\">\n", action.c_str());
}

void end_form()
{
	puts("</FORM>");
}

string edittext(const string& name, const string& value = "", int size)
{
	string tag = (string)"<INPUT TYPE=\"text\" NAME=\"" + name + "\"";
	if (value != "") {
		tag += (string)" VALUE=\"" + value + "\"";
	}
	if (size) {
		tag += format(" SIZE=\"%d\"", size);
	}

	tag += ">";
	return tag;
}

string editpassword(const string& name, const string& value = "")
{
	string tag = (string)"<INPUT TYPE=\"password\" NAME=\"" + name + "\"";
	if (value != "") {
		tag += (string)" VALUE=\"" + value + "\"";
	}
	tag += ">";
	return tag;
}

string textarea(const string& name, const string& value, int rows, int cols)
{
	return 
		format("<TEXTAREA NAME=\"%s\" rows=\"%d\" cols=\"%d\">%s</TEXTAREA>", 
			name.c_str(), rows, cols, value.c_str());
}

string hidden(const string& name, const string& value = "")
{
	string tag = (string)"<INPUT TYPE=\"hidden\" NAME=\"" + name + "\"";
	if (value != "") {
		tag += (string)" VALUE=\"" + value + "\"";
	}
	tag += ">";
	return tag;
}

string submit(const string& face = "")
{
	string tag = (string)"<INPUT TYPE=\"SUBMIT\"";
	if (face != "") {
		tag += (string)"VALUE=\"" + face + "\"";
	}
	tag += ">";
	return tag;
}

string hyperlink(const string& url, const string& word)
{
	return format("<A href=\"%s\">%s</A>", url.c_str(), word.c_str());
}

string font(const string& fonttype, const string& word)
{
	if (fonttype == "") return word;
	return format("<FONT %s>%s</FONT>", fonttype.c_str(), word.c_str());
}

void print(const string& msg)
{
	printf("%s", msg.c_str());
}

